package org.apidesign.apifest08.currency;

import java.util.Currency;
import java.util.Date;
import org.apidesign.apifest08.currency.Convertor.ConversionRequest;
import org.apidesign.apifest08.currency.Convertor.ConversionResult;

/**
 * This {@link Convertor} delegates to an underlying one, provided that the request specifies time between from (included) and till (excluded).
 * Otherwise it just refuses to convert.
 * @author jdvorak
 */
public class TimeRangeSpecificConvertor extends DelegatingConvertor {

    private final Date from;
    private final Date till;

    /**
     * A new time range specific convertor.
     * @param old the underlying convertor one delegates to
     * @param from the beginning of the time interval
     * @param till the end of the time interval
     * @throws IllegalArgumentException unless from comes before till
     */
    public TimeRangeSpecificConvertor( final Convertor old, final Date from, final Date till ) {
        super( old );
        this.from = from;
        this.till = till;
        if (! from.before( till ) ) {
            throw new IllegalArgumentException( "from must come before till" );
        }
    }

    /**
     * The beginning of the time interval.
     */
    public Date getFrom() {
        return from;
    }

    /**
     * The end of the time interval.
     */
    public Date getTill() {
        return till;
    }

    /**
     * The conversion method.
     * Takes a {@link TimeSpecificConversionRequest}, other {@link ConversionRequest}s will result in an exception being thrown.
     * @param req the request; must not be null; must be {@link TimeSpecificConversionRequest} or a subclass thereof
     * @return the response
     * @throws IllegalRequestSubtypeException iff req is not instance of {@link TimeSpecificConversionRequest}
     */
    @Override
    public ConversionResult convert( final ConversionRequest req ) {
        if ( req instanceof TimeSpecificConversionRequest ) {
            final Date time = ( (TimeSpecificConversionRequest) req ).getTime();
            if ( ( from == null || !from.after(time) ) && ( till == null || time.before(till) ) ) {
                return super.convert( req );
            } else {
                return new ConversionResult( null );
            }
        } else {
            throw new IllegalRequestSubtypeException( this.getClass().getName() + ".convert() requires a TimeSpecificConversionRequest to be passed in" );
        }
    }

    /**
     * The request for converting a monetary amount into another currency using the conditions that apply at a particular time.
     * Immutable.
     */
    public static class TimeSpecificConversionRequest extends Convertor.ConversionRequest {

        private final Date time;

        /**
         * A request to convert srcAmount into tgtCurrency at the current time.
         * @param srcAmount the source amount; must not be null
         * @param tgtCurrency the currency we want it in afterwards; must not be null
         */
        public TimeSpecificConversionRequest( final MonetaryAmount srcAmount, final Currency tgtCurrency ) {
            this( srcAmount, tgtCurrency, new Date() );
        }

        /**
         * A request to convert srcAmount into tgtCurrency at given time.
         * @param srcAmount the source amount; must not be null
         * @param tgtCurrency the currency we want it in afterwards; must not be null
         * @param time the time instant when the conversion is to be carried out
         */
        public TimeSpecificConversionRequest( final MonetaryAmount srcAmount, final Currency tgtCurrency, final Date time ) {
            super( srcAmount, tgtCurrency );
            this.time = time;
            if ( time == null ) {
                throw new NullPointerException( "The time of conversion" );
            }
        }

        /**
         * The time as of which the conversion is to be carried out.
         */
        public Date getTime() {
            return time;
        }

    }

}
