
package org.apidesign.apifest08.currency;

import java.util.Currency;

/**
 * Desription of currency. 
 * 
 * Java has similar class {@link java.util.Currency}, but original class is not flexible
 * enough, we use our own implementation of currency.
 *
 * @author arnostvalicek
 */
public class ConvertorCurrency {
    private String currencyCode;
    private int fractionDigits;


    /**
     * Static method providing instance of <code>ConvertorCurrency</code> based on currency code.
     * <p>
     * <code>currencyCode</code> is one of codes supported by Java class {@link java.util.Currency}.
     * 
     * @param currencyCode Code of required currency.
     * @return Returns required <code>ConvertorCurrency</code>
     */
    public static ConvertorCurrency getInstance(String currencyCode) {
        ConvertorCurrency convertorCurrency = new ConvertorCurrency();
        Currency currency = Currency.getInstance(currencyCode);
        convertorCurrency.setCurrencyCode(currency.getCurrencyCode());
        convertorCurrency.setDefaultFractionDigits(currency.getDefaultFractionDigits());
        return convertorCurrency;
    }
    
    /**
     * Static method providing instance of <code>ConvertorCurrency</code> base of currency code.
     * 
     * @param currencyCode Code of required currency.
     * @param fractionDigits Number of fraction digits for currency.
     * @return Returns required <code>ConvertorCurrency</code>
     * @since version4
     */
    public static ConvertorCurrency getInstance(String currencyCode, int fractionDigits) {
        ConvertorCurrency convertorCurrency = new ConvertorCurrency();
        convertorCurrency.setCurrencyCode(currencyCode);
        convertorCurrency.setDefaultFractionDigits(fractionDigits);
        return convertorCurrency;
    }

    /**
     * Gets the default number of fraction digits used with this currency. For example, the default number of fraction digits for the Euro is 2, while for the Japanese Yen it's 0.
     * @return Returns the default number of fraction digits used with this currency.
     */
    public int getDefaultFractionDigits() {
        return fractionDigits;
    }
    
    private void setDefaultFractionDigits(int value) {
        this.fractionDigits = value;
    }
    
    /**
     * Get currency code.
     * @return Returns currency code.
     */
    public String getCurrencyCode() {
        return currencyCode;
    }
    
    private void setCurrencyCode(String value) {
        currencyCode = value;
    }

    @Override
    public boolean equals(Object obj) {
        boolean result;
        if (obj instanceof ConvertorCurrency) {
            ConvertorCurrency that = (ConvertorCurrency) obj;
            result = getCurrencyCode().equals(that.getCurrencyCode())  && getDefaultFractionDigits()==that.getDefaultFractionDigits();
        } else {
            result = false;
        }
        return result;
    }

    @Override
    public int hashCode() {
        return currencyCode==null ? 47/*??*/ : currencyCode.hashCode();
    }
        

    @Override
    public String toString() {
        return  "ConvertorCurrency[" + (currencyCode != null ? currencyCode : "NO-BASE-CURRENCY")+"]";
    }
   
}